package com.docus.webservice.utils;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.*;
import java.util.Date;
import java.util.Map;

/**
 * json工具类
 */
public class JsonUtils {

    /**
     * 读取json文件方法
     * @param fileName：json文件存在的本地地址
     * @return
     */
    public static String readJsonFile(String fileName) {
        String jsonStr = "";
        Reader reader=null;
        FileReader fileReader=null;
        try {
            File jsonFile = new File(fileName);
            fileReader = new FileReader(jsonFile);
            reader = new InputStreamReader(new FileInputStream(jsonFile),"utf-8");
            int ch = 0;
            StringBuffer sb = new StringBuffer();
            while ((ch = reader.read()) != -1) {
                sb.append((char) ch);
            }
            fileReader.close();
            reader.close();
            jsonStr = sb.toString();
            return jsonStr;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }finally {
            try {
                reader.close();
                fileReader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private final static String STRINGCLASS = "";
    private final static Long LONGCLASS = new Long(0);
    private final static Date DATECLASS = new Date();

    /**
     * 将Map集合中的数据封装到JavaBean对象中
     * map的key为全小写
     * bean属性驼峰
     * @param map           集合
     * @param classType     封装javabean对象
     * @return
     * @throws Exception
     */
    public static <T> T map2bean(Map<String, Object> map, Class<T> classType) throws Exception {
        // 1 采用反射动态创建对象
        T obj = classType.newInstance();

        // 2 获取对象字节码信息,不要Object的属性
        BeanInfo beanInfo = Introspector.getBeanInfo(classType,Object.class);

        // 3 获取bean对象中的所有属性
        PropertyDescriptor[] list = beanInfo.getPropertyDescriptors();
        for (PropertyDescriptor pd : list) {
            // 3.1 获取属性名
            String key = pd.getName();

            // 3.2 获取属性值:属性值在map中全部存的小写
            Object value = map.get(key.toLowerCase());

            // 3.3 转换数据类型
            if (value != null && StringUtils.isNotBlank(value.toString())) {
                Class<?> propertyType = pd.getPropertyType();
                if (propertyType.isInstance(STRINGCLASS)) { // 转换成String类型
                    value = (String) value;
                }
                if (propertyType.isInstance(LONGCLASS)) { // 转换成Long类型
                    value = Long.parseLong((String) value);
                }
                if (propertyType.isInstance(DATECLASS)) {// 转换成日期类型
                    value = DateUtils.parseDate((String) value);
                }

                pd.getWriteMethod().invoke(obj, propertyType.cast(value));     //调用属性setter()方法,设置到javabean对象当中
            }
        }
        return obj;
    }

}
